"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const __1 = require("..");
/**
 * Class responsible for validating enum fields.
 * @typeparam T The type of the enum.
 * @typeparam K The type of the enum keys.
 * @typeparam V The type of the enum values.
 */
class EnumField extends __1.Field {
    constructor(enumType, description) {
        super(description);
        this.enumType = enumType;
        this.valueToKey = __1.transformObject(enumType, (value, key) => [value, key]);
    }
    /**
     * Creates a new enum field for the given enum type.
     * @param enumType The type of the enum.
     * @param description The description of the field.
     * @typeparam T The type of the enum.
     * @typeparam K The type of the enum keys.
     * @typeparam V The type of the enum values.
     * @return The created field.
     */
    static of(enumType, description) {
        return new EnumField(enumType, description);
    }
    /**
     * Parses and validates the given value, throwing an error if the value is invalid.
     * @param key The attribute name of the field in the model.
     * @param value The value to parse. It may be either the internal or the external type, but it should not be null.
     * @return The parsed value.
     * @throws ValidationError
     */
    set(key, value) {
        if (this.valueToKey.hasOwnProperty(value)) {
            return value;
        }
        if (this.enumType.hasOwnProperty(value)) {
            return this.enumType[value];
        }
        throw __1.invalidEnumValue(key, value);
    }
    /**
     * Converts the value from the internal format to the external format.
     * @param key The attribute name of the field in the model.
     * @param value The value to parse.
     * @return The parsed value.
     * @throws ValidationError
     */
    serialize(key, value) {
        return this.valueToKey[value.toString()];
    }
}
exports.default = EnumField;
//# sourceMappingURL=enum-field.js.map