/**
 * The base class for all fields.
 * @typeparam TExternal The type used when the field is serialized.
 * @typeparam TInternal The type used when the field is used internally (it must match the class property).
 */
export default abstract class Field<TExternal, TInternal = TExternal> {
    private descriptionText;
    private options;
    /**
     * Creates a new field.
     * @param descriptionText The description of the field.
     * @param options Settings for the field.
     */
    protected constructor(descriptionText: string, options?: {
        required?: boolean;
    });
    /**
     * Gets the description of the field.
     */
    readonly description: string;
    /**
     * Marks this field as required.
     * @return this
     */
    asRequired(): this;
    /**
     * Gets the value of the field or its default value. This raises an error if the field is required and has no value yet.
     * @param key The attribute name of the field in the model.
     * @param value The value of the field.
     * @return The parsed value.
     */
    get(key: string, value?: TInternal | undefined): TInternal | undefined;
    /**
     * Parses and validates the given value, throwing an error if the value is invalid.
     * @param key The attribute name of the field in the model.
     * @param value The value to parse. It may be either the internal or the external type, but it should not be null.
     * @return The parsed value.
     * @throws ValidationError
     */
    abstract set(key: string, value: TExternal | TInternal): TInternal | null;
    /**
     * Converts the value from the internal format to the external format.
     * @param key The attribute name of the field in the model.
     * @param value The value to parse.
     * @return The parsed value.
     * @throws ValidationError
     */
    abstract serialize(key: string, value: TInternal): TExternal;
}
