"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const __1 = require("..");
class StringField extends __1.Field {
    /**
     * Creates a new string field.
     * @param description The description of the field.
     * @param regex An optional regex to validate the field with.
     */
    constructor(description, regex = null) {
        super(description);
        this.regex = regex;
    }
    /**
     * Creates a new string field.
     * @param description The description of the field.
     * @return The created field.
     */
    static create(description) {
        return new StringField(description);
    }
    /**
     * Adds a regex validation to the string field.
     * @param regex The regex to validate the field with.
     * @return this
     */
    withRegex(regex) {
        this.regex = regex;
        return this;
    }
    /**
     * Parses and validates the given value, throwing an error if the value is invalid.
     * @param key The attribute name of the field in the model.
     * @param value The value to parse. It may be either the internal or the external type, but it should not be null.
     * @return The parsed value.
     * @throws ValidationError
     */
    set(key, value) {
        if (typeof value === 'string') {
            if (this.regex && !this.regex.test(value)) {
                throw __1.invalidFormat(key, value, this.regex.toString());
            }
            return value.trim();
        }
        throw __1.invalidType(key, value);
    }
    /**
     * Converts the value from the internal format to the external format.
     * @param key The attribute name of the field in the model.
     * @param value The value to parse.
     * @return The parsed value.
     * @throws ValidationError
     */
    serialize(key, value) {
        return value;
    }
}
exports.default = StringField;
//# sourceMappingURL=string-field.js.map