"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const __1 = require("..");
/**
 * Models are responsible for defining their own fields.
 */
class Model {
    constructor() {
        /** The data of the model */
        this.$data = {};
        this.defineProperties();
    }
    /**
     * Adds a new field to the class.
     * @param key The key of the field.
     * @param field The field.
     */
    addField(key, field) {
        this.$fields = this.$fields || {};
        this.$fields[key] = field;
    }
    /**
     * Defines the properties based on the fields of the class.
     */
    defineProperties() {
        // hide the data field from Object.keys()
        Object.defineProperty(this, '$data', { enumerable: false, writable: false, value: this.$data });
        Object.defineProperties(this, __1.mapObject(this.$fields, (field, key) => ({
            /**
             * Gets the value of the field, delegating validation to the field.
             * @return The value of the field.
             */
            get() {
                return field.get(key, this.$data[key]);
            },
            /**
             * Sets the value of the field, delegating conversion and validation to the field.
             * @param value The value to set.
             */
            set(value) {
                // call the get method when the new value is empty, to allow for validations / default values
                this.$data[key] = value == null ? field.get(key) : field.set(key, value);
            },
            enumerable: true,
            configurable: false,
        })));
    }
    /**
     * Populates this model with all the given properties. Non-existent fields are ignored.
     * @param properties The properties to set.
     * @return this.
     */
    populate(properties = {}) {
        __1.forEachObject(properties, (value, key) => this.$fields[key] && (this[key] = value));
        return this;
    }
    /**
     * Validates the schema. Override this method to add extra validations.
     * @return this.
     */
    validate() {
        __1.forEachObject(this.$fields, (field, key) => this[key] = this[key]);
        return this;
    }
    /**
     * Validates the schema and serializes it to the output format.
     * @return The serialized properties.
     */
    serialize() {
        this.validate();
        return __1.mapObject(this.$fields, (field, key) => {
            const value = this[key];
            return value == null ? null : field.serialize(key, value);
        });
    }
}
exports.default = Model;
//# sourceMappingURL=model.js.map