"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
/**
 * Runs the given function to each item in the object.
 * @param obj The object to iterate through.
 * @param fn The function to call.
 * @typeparam TObj The type of the parameter `obj`.
 * @typeparam TIn The the type of the properties for which the @param fn will be applied.
 * @typeparam TOut The type of the output of the parameter `fn`.
 * @typeparam TKey The keys of type parameter `TObj`
 */
function forEachObject(obj, fn) {
    Object.keys(obj).forEach(key => fn(obj[key], key));
}
exports.forEachObject = forEachObject;
/**
 * Creates a new object with the results of calling the given function for each item.
 * If the function returns null or undefined, the resulting object will not have that key.
 * @param obj The object to iterate through.
 * @param fn The function to call.
 * @typeparam TObj The type of the parameter `obj`.
 * @typeparam TIn The the type of the properties for which the @param fn will be applied.
 * @typeparam TOut The type of the output of the parameter `fn`.
 * @typeparam TKey The keys of type parameter `TObj`
 * @return An object with the results of the function calls.
 */
function mapObject(obj, fn) {
    const result = {};
    forEachObject(obj, (oldValue, key) => {
        const newValue = fn(oldValue, key);
        if (newValue != null) {
            result[key] = newValue;
        }
    });
    return result;
}
exports.mapObject = mapObject;
/**
 * Creates a new object with the results of calling the given function for each item.
 * This is similar to {@link mapObject} but it allows for new keys in the resulting object.
 * @param obj The object to iterate through.
 * @param fn The function to call. It should return a tuple with the new key and new value.
 * @typeparam TObjIn The type of the parameter `obj`.
 * @typeparam TIn The the type of the properties for which the @param fn will be applied.
 * @typeparam TOut The type of the output of the parameter `fn`.
 * @typeparam TKeyIn The keys of type param `TObjIn`
 * @typeparam TKeyOut The keys of the output.
 * @return An object with the results of the function calls.
 */
function transformObject(obj, fn) {
    const result = {};
    forEachObject(obj, (value, key) => {
        const [newKey, newValue] = fn(value, key);
        result[newKey] = newValue;
    });
    return result;
}
exports.transformObject = transformObject;
//# sourceMappingURL=objects.js.map